<section id="amps" class="plugin">
    <div class="table">
        <div class="table-row" ng-repeat="process in vm.processes">
            <div class="table-cell text-left" ng-class="vm.getDescriptionDecoration(process)">{{ process.name }}</div>
            <div class="table-cell text-left">{{ process.count }}</div>
            <div class="table-cell text-left process-result">{{ process.result }}</div>
        </div>
    </div>
</section>
