<div class="table-row" ng-repeat="port in vm.ports">
    <div class="table-cell text-left">{{(port.description ? port.description : port.host + ' ' + port.port) | min_size:
        20}}
    </div>
    <div class="table-cell"></div>
    <div ng-switch="port.status" ng-class="vm.getPortDecoration(port)" class="table-cell" ng-if="port.host">
        <span ng-switch-when="null">Scanning</span>
        <span ng-switch-when="false">Timeout</span>
        <span ng-switch-when="true">Open</span>
        <span ng-switch-default>{{port.status * 1000.0 | number:0}}ms</span>
    </div>
    <div ng-switch="port.status" ng-class="vm.getWebDecoration(port)" class="table-cell" ng-if="port.url">
        <span ng-switch-when="null">Scanning</span>
        <span ng-switch-when="Error">Error</span>
        <span ng-switch-default>Code {{ port.status }}</span>
    </div>
</div>