<div class="table-row" ng-if="vm.hotspots.length > 0">
    <div class="table-cell text-left title">WIFI</div>
    <div class="table-cell"></div>
    <div class="table-cell">dBm</div>
</div>
<div class="table-row" ng-repeat="hotspot in vm.hotspots">
    <div class="table-cell text-left">{{ hotspot.ssid|limitTo:20 }} <span ng-if="hotspot.encrypted">{{ hotspot.encryption_type }}</span>
    </div>
    <div class="table-cell"></div>
    <div class="table-cell" ng-class="vm.getDecoration(hotspot, 'signal')">{{ hotspot.signal }}</div>
</div>
