<section id="percpu" class="plugin">
    <div class="table" ng-repeat="cpus in vm.cpusChunks">
        <div class="table-row">
            <div class="table-cell text-left title"><span ng-if="$first">PER CPU</span></div>
            <div class="table-cell" ng-repeat="percpu in cpus track by percpu.number">{{ percpu.total }}%</div>
        </div>
        <div class="table-row">
            <div class="table-cell text-left">user:</div>
            <div class="table-cell" ng-repeat="percpu in cpus track by percpu.number"
                 ng-class="vm.getUserAlert(percpu)">
                {{ percpu.user }}%
            </div>
        </div>
        <div class="table-row">
            <div class="table-cell text-left">system:</div>
            <div class="table-cell" ng-repeat="percpu in cpus track by percpu.number"
                 ng-class="vm.getSystemAlert(percpu)">
                {{ percpu.system }}%
            </div>
        </div>
        <div class="table-row">
            <div class="table-cell text-left">idle:</div>
            <div class="table-cell" ng-repeat="percpu in cpus track by percpu.number">{{ percpu.idle }}%</div>
        </div>
        <div class="table-row" ng-if="vm.cpus[0].iowait">
            <div class="table-cell text-left">iowait:</div>
            <div class="table-cell" ng-repeat="percpu in cpus track by percpu.number"
                 ng-class="vm.getSystemAlert(percpu)">
                {{ percpu.iowait }}%
            </div>
        </div>
        <div class="table-row" ng-if="vm.cpus[0].steal">
            <div class="table-cell text-left">steal:</div>
            <div class="table-cell" ng-repeat="percpu in cpus track by percpu.number"
                 ng-class="vm.getSystemAlert(percpu)">
                {{ percpu.steal }}%
            </div>
        </div>
    </div>
</section>
